#    most of this task is now taken from the standard application

set VERSION "VHSADC"

#    put some default environment variables before loading common 

set env(RDOUThdwre) cPCI
set env(RDOUTserver) DAQ
set env(ServerAccessVersion) 2

loadsource  sys.common
loadsource  sys.WWW


set SpectrumServerList {}
set SpectrumServer ""

set Supported_Modules {}

set DADirectory $Win32Directory
set DAProgram VHSADC_RTDEXexecV2.exe

lappend setuplist [list "Show histogram name translations" "Show_Xlate"]
lappend setuplist [list "Edit histogram translation file"  "edit_xlate"]

lappend actionlist [list "Raw Data Display"  "Dump_Raw_Data"]

lappend registerlist [list NetVar.DACQ.RunControl     generic  "-n DACQ_RunControl -t int"]

for {set i 1} {$i <= 7} {incr i} {
    lappend registerlist [list NetVar.DACQ.Stats$i  generic  "-n DACQ_Stats -t block -a yes -i [expr 512*$i] -l 512 -f d"]
    lappend registerlist [list NetVar.DACQ.Rates$i  generic  "-n DACQ_Rates -t block -a yes -i [expr 512*$i] -l 512 -f d"]
}

set frametitle "MIDAS Data Acquisition Experiment Control (VHSADC)"

set TSPresent 1
set TSEnable 0
set txEnable 0

set histEnable 1
set HistMode 1

set TestBuffer ""

proc Dump_Raw_Data {} {

   set info "module 1\n"

   execute-command 19 "Getting Raw Data" "14 1" 2
   set data [read-register NetVar.EXEC.Response]

     foreach word $data {
         set word $word
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "\n"
             continue
         }
         append info "$word "
     }

   append info "module 2\n"

   execute-command 19 "Getting Raw Data" "14 2" 2
   set data [read-register NetVar.EXEC.Response]

     foreach word $data {
         set word $word
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "\n"
             continue
         }
         append info "$word "
     }

   midas-information "$info"
}

proc Dump_Raw_VME_Data {} {Dump_Raw_Data}


proc Custom_Start {} {
    global Options

   if {![info exists Options(VHSADC_GroupBase)]} {set Options(VHSADC_GroupBase) 1}

   if {![info exists Options(Include.VHSADC)]} {set Options(Include.VHSADC) 1}
   if {![info exists Options(VHSADC_Hist_Trace_Enable)]} {set Options(VHSADC_Hist_Trace_Enable) 0}
   if {![info exists Options(VHSADC_Hist_Energy_Enable)]} {set Options(VHSADC_Hist_Energy_Enable) 1}
   if {![info exists Options(VHSADC_Hist_BaseLine_Enable)]} {set Options(VHSADC_Hist_BaseLine_Enable) 1}
   if {![info exists Options(VHSADC_Hist_Time_Enable)]} {set Options(VHSADC_Hist_Time_Enable) 0}
   if {![info exists Options(VHSADC_Hist_TimeError_Enable)]} {set Options(VHSADC_Hist_TimeError_Enable) 0}
   if {![info exists Options(VHSADC_Hist_RAW_Enable)]} {set Options(VHSADC_Hist_RAW_Enable) 0}

   update_Options
}

proc Custom_PreSetup {args} {

     inform vhsadc do_Start 0    ;# in case VHSADC Web Service restarted 

     while {[inform vhsadc inquire started] != 2} {
         midas-warning "You must first start the VHSADC Client and Select and Initialise the VHSADC Web Service\nThen continue."
     }

     inform vhsadc do_Load 0

    ClearConfig
    do_pgmoption POLLTICKS 100
}

proc Custom_PostSetup {args} {

    set-footer "Requesting restore of VHSADC parameters"
    inform vhsadc Restore
    clear-footer
}

proc Stop_Electronics {} {
    inform vhsadc do_acqGO 0
}

proc Go_Electronics {} {
    inform vhsadc do_acqGO 1
}

proc Reset_Electronics {} {
       global RegisterServer
}

proc click-histenable {v} {
    global RDOUThdwre RDOUTEnable histEnable
    if {$RDOUTEnable == 0} then {set  histEnable 0} else {set histEnable $v}
    if {$RDOUTEnable != 0} then {write-register NetVar.DACQ.Chistenable [expr $histEnable * 10]}
    redisplay-histenable
}

proc create_spectra {} {
   global SpectrumServer HistMode EGreport
   global Options

   if {$HistMode == 0} {return}
   
   set-footer "Creating spectra"
   set-busy

   set EGreport_SAVE $EGreport
   set EGreport -1

   Make_Xlate $SpectrumServer

   set-footer "Creating Stat"
   set range $Options(Stat.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]

   set-footer "Creating Rate"
   set range $Options(Rate.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

   map_spectrum "Stat 0 0 0 0 0"      ;#  map as group 0 item 0

   set-footer "Creating VHSADC spectra"

   create_VHSADC_spectra
   map_spectra

   set EGreport $EGreport_SAVE

   clear-busy
   clear-footer
}



proc create_VHSADC_spectra {} {
   global SpectrumServer
   global Options


#   for the VHSADC 16 channel module

   set maxmodule [inform vhsadc inquire maxboards]
   set maxadc 16

   for {set i 1} {$i <= $maxmodule} {incr i} {

     set maxadc [inform vhsadc get_numchans [expr $i-1]]
     set groupbase $Options(VHSADC_GroupBase)

#    histogram used to display the digital trace
#    need to obtain trace length from the board

     set framesize [inform vhsadc get_framesize [expr $i-1]]

     if {$Options(VHSADC_Hist_Trace_Enable) == 1 && $framesize >= 24} {

        for {set j 1} {$j <= $maxadc} {incr j} {    ;#  normal histogram to show trace buffer

           set spectrumname [SpectrumName vhsadc#$i.ch$j]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $framesize :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           }
        }
     }

     if {$Options(VHSADC_Hist_RAW_Enable) == 1} {

        set vl [inform vhsadc get_datasize [expr $i-1]]

        if {$vl != 2 && $vl != 4} {
           midas-warning "VHS-ADC module $i not correctly setup; data size = $vl"
           set vl 4     ;# use default
        }

        if {$vl == 4} {set range 2048} else {set range 1024}

 #       for {set j 1} {$j <= $maxadc} {incr j} {     ;#   histogram to show RAW data buffer

           set spectrumname [SpectrumName vhsadc#$i.R]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           }
 #       }
     }

     if {$Options(VHSADC_Hist_Energy_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {   

#    histograms for the computed energy

           set spectrumname [SpectrumName vhsadc#$i.E[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           } 
        }
     }

     if {$Options(VHSADC_Hist_BaseLine_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {

#    histograms for the base line

           set spectrumname [SpectrumName vhsadc#$i.B[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           }
        }
     }

     if {$Options(VHSADC_Hist_Time_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {

#    histograms for the Time

           set spectrumname [SpectrumName vhsadc#$i.TH[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           }

           set spectrumname [SpectrumName vhsadc#$i.TM[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           }

           set spectrumname [SpectrumName vhsadc#$i.TL[set j]]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           }
        }
     }

     if {$Options(VHSADC_Hist_TimeError_Enable) == 1} {

#    histograms for the TimeStamp Error

           set spectrumname [SpectrumName vhsadc#$i.TE]
           set-footer "Creating $spectrumname"
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $spectrumname"
              clear-footer
              error $m
           }
     }

   }

}

proc map_spectra {} {
   global env SpectraDefs

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

   map_VHSADC_spectra

   do_UpLoadSpectraDefs

   execute-command 4 "Mapping spectra" [file join c:/ temp VHSADCServer VHSADCSpectraDefs.dat]
}

#    This procedure creates a file containing list of spectra to be mapped

proc map_VHSADC_spectra {} {
   global SpectrumServer
   global Options
   global env tcl_platform
   global SpectraDefs

   if {![info exists env(MIDASTempDir)] || ![file exists $env(MIDASTempDir)]} then {
       switch $tcl_platform(platform) {
          windows {set env(MIDASTempDir) [file join c:/ temp]}
          default {set env(MIDASTempDir) \tmp}
       }
   }

   catch {file mkdir $env(MIDASTempDir}
   set SpectraDefs [file join $env(MIDASTempDir) VHSADCSpectraDefs.dat]

      set z [catch {open $SpectraDefs w} f]
      if {$z != 0} then {
          midas-warning "Cannot open file $SpectraDefs\n$f"
          return
      }

#   for the VHSADC 16 channel module

   set maxmodule [inform vhsadc inquire maxboards]
   set maxadc 16

   for {set i 1} {$i <= $maxmodule} {incr i} {

     set maxadc [inform vhsadc get_numchans [expr $i-1]]
     set groupbase $Options(VHSADC_GroupBase)

     set framesize [inform vhsadc get_framesize [expr $i-1]]

     if {$Options(VHSADC_Hist_Trace_Enable) == 1 && $framesize >= 24} {

        for {set j 1} {$j <= $maxadc} {incr j} {    ;#  normal histogram to show trace buffer

           set spectrumname [SpectrumName vhsadc#$i.ch$j]
#           set-footer "Mapping $spectrumname"
#           map_spectrum  "$spectrumname [expr 256 + $groupbase + (($i-1)*16) + ($j-1)] 0 0 0"
           puts $f "$spectrumname [expr 256 + $groupbase + (($i-1)*16) + ($j-1)] 0 0 0"
        }
     }

     if {$Options(VHSADC_Hist_RAW_Enable) == 1} {

 #       for {set j 1} {$j <= $maxadc} {incr j} {     ;#   histogram to show RAW data buffer

           set spectrumname [SpectrumName vhsadc#$i.R]
#           set-footer "Mapping $spectrumname"
#           map_spectrum  "$spectrumname [expr 384 + $groupbase + (($i-1)*16) + ($j-1)] 0 0 0"
           puts $f "$spectrumname [expr 384 + $groupbase + (($i-1)*16)] 0 0 0"
 #       }
     }

     if {$Options(VHSADC_Hist_Energy_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {   

           set spectrumname [SpectrumName vhsadc#$i.E$j]
#           set-footer "Mapping $spectrumname"
#           map_spectrum  "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 0 0 0"         ;#  item 0
           puts $f "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 0 0 0"
        }
     }

     if {$Options(VHSADC_Hist_BaseLine_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {

           set spectrumname [SpectrumName vhsadc#$i.B$j]
#           set-footer "Mapping $spectrumname"
#           map_spectrum  "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 1 0 0"         ;#  item 1
           puts $f "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 1 0 0"
        }
     }

     if {$Options(VHSADC_Hist_Time_Enable) == 1} {

        for {set j 1} {$j <= $maxadc} {incr j} {

           set spectrumname [SpectrumName vhsadc#$i.TH$j]
#           set-footer "Mapping $spectrumname"
#           map_spectrum  "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 4 0 0"         ;#  item 4
           puts $f "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 4 0 0"

           set spectrumname [SpectrumName vhsadc#$i.TM$j]
#           set-footer "Mapping $spectrumname"
#           map_spectrum  "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 3 0 0"         ;#  item 3
           puts $f "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 3 0 0"

           set spectrumname [SpectrumName vhsadc#$i.TL$j]
#           set-footer "Mapping $spectrumname"
#           map_spectrum  "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 2 0 0"         ;#  item 2
           puts $f "$spectrumname [expr $groupbase + (($i-1)*16) + ($j-1)] 2 0 0"
        }
     }

     if {$Options(VHSADC_Hist_TimeError_Enable) == 1} {

           set spectrumname [SpectrumName vhsadc#$i.TE]
           puts $f "$spectrumname [expr 128 + $groupbase + ($i-1)] 0 0 2"        ;#  item 0 - shift 2
     }

   }

   close $f
}

proc do_UpLoadSpectraDefs {} {
    upvar #0 SpectraDefs file

        set z [catch {inform vhsadc VHSADCClient_VHSADCUpLoad $file} rc]
        if {$z == 0} {
            set-footer "Uploaded file $file successful"
        } else {
            set-footer "Uploading file $file failed\n$rc"
        }
}



#    additions taken from grt4 for Liverpool scanning table


set scantable 0

proc do_scantable {w v} {
    global scantable
 
    set scantable [expr $v*2]
    execute-command 16 "" "$scantable 0 0 0 0 0" 2
}

proc enable_scanning {} {execute-command 16 "" "3 1 2 3 4 5" 2}



#   :choice-stack FORMATOPTION  :label "Format Option" :strings [list GANIL GSI MIDAS GREAT]
