#    Control module for the TDR Interface Card (TDRI)

set frame  TDRI
set title "TDR Interface Card"
set icon   TDRI

set started 0

set TDRIServerList {}
set TDRIServer "select a server"
variable WebServer


proc bgerror args {
    global errorInfo tkerrorInfo tkskiperror
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    if {![info exists FaultCode] || $FaultCode == "NONE"} {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n[lindex $args 0]" "error" 0 "continue" "Stack Trace"} button]
    } else {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo" \
                       "error" 0 "continue"} button]
    }
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}

proc click_redisplay {args} {Redisplay}

proc click_dac1sel {i args} {

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}

    set vl [expr ($m & 0xfc) + [first $i]]
    set rc [catch {TDRIAccessClient__WriteControl $vl} m]
    if {$rc != 0} {bgerror; return}

}

proc click_dac2sel {i args} {

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}

    set vl [expr ($m & 0xf3) + ([first $i] << 2)]
    set rc [catch {TDRIAccessClient__WriteControl $vl} m]
    if {$rc != 0} {bgerror; return}

}

proc click_veto {i args} {

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}

    set vl [expr ($m & 0xef) + ([first $i] << 4)]
    set rc [catch {TDRIAccessClient__WriteControl $vl} m]
    if {$rc != 0} {bgerror; return}

}

proc click_fp {i args} {

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}

    set vl [expr ($m & 0xdf) + ([first $i] << 5)]
    set rc [catch {TDRIAccessClient__WriteControl $vl} m]
    if {$rc != 0} {bgerror; return}

}

proc Redisplay {} {

    set rc [catch {TDRIAccessClient__Version} m]
    if {$rc != 0} {bgerror; return}
    :modify VERS :value $m

    set rc [catch {TDRIAccessClient__Status} m]
    if {$rc != 0} {bgerror; return}

    :modify ST0 :value [expr $m & 1]
    :modify VETO :value [expr ($m >> 1) & 1]

    set rc [catch {TDRIAccessClient__ReadControl} m]
    if {$rc != 0} {bgerror; return}
    :modify DAC1SEL :value [expr $m & 3]
    :modify DAC2SEL :value [expr ($m >> 2) & 3]
    :modify VETO    :value [expr ($m >> 4) & 1]
    :modify FP      :value [expr ($m >> 5) & 1]

    set-footer "Control = [format "0x%x" $m]"

}

proc click arg {
    global actonall

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      WEBSERVERT    {click-server $v}
      WEBSERVERC    {click-server $i}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p [list $v $i]} else {insert-debug "$arg"}
      }
   }
}

proc start args {
   global started
   global WebServer TDRIServerList Options

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1

   load_Options

   if {![info exists Options(WebServer)]}     {set Options(WebServer) "localhost"}
   if {![info exists Options(AutoSelect)]}    {set Options(AutoSelect) "1"}

   update_Options

   foreach option [list WebServer] {
       global $option
       set $option $Options($option)
   }

   set TDRIServerList [list $WebServer]    ;# put default into the menu
   :modify WEBSERVERC :strings $TDRIServerList :value [lsearch $TDRIServerList $WebServer]

   if {$Options(AutoSelect) == 1} {click-server $WebServer}

}

proc click-server {v} {
    global env TDRIServerList WebServer TDRIAccessServerHost

    set WebServer $v
    set TDRIServerList [list $WebServer]    ;# put default into the menu
    :modify WEBSERVERT :value $WebServer
    :modify WEBSERVERC :strings $TDRIServerList :value [lsearch $TDRIServerList $WebServer]

    Save_Options

   set TDRIAccessServerHost $WebServer
   package require TDRIAccessClient
   TDRIAccessClient_Configure

   TDRIAccessClient__Init

    Redisplay
}

proc make-frame {} {
   global env

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :text WEBSERVERT         :label "TDRI WebServer" :w 20  :value "select a server"
    :choice-stack WEBSERVERC :label "" :strings "xxx.xxx.xxx.xxx" 
    :hskip 20
    :next-row T
    :button OPTIONS :label "Reload Options" :command click-options

  :panel P1 :layout T :padx 5 :pady 5
    :layout V; :col-gap 10; :row-gap 10

    :choice DAC1SEL :label "Select input for DAC1 visualisation" :strings [list 1 2 3 4]  :valuex 35c
    :choice DAC2SEL :label "Select input for DAC2 visualisation" :strings [list 1 2 3 4]  :valuex 35c
    :choice VETO    :label "Veto mode select"    :strings [list loopback normal] :valuex 35c
    :choice FP      :label "TDR Clk Sync select" :strings [list off on] :valuex 35c

  :panel P2 :layout T :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :checkbox ST0   :label "High Temperature" :deaf T
    :text VERS      :label "firmware version" :w 6 :deaf T

  :panel BUTTONS :after footer :layout B :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

     :button REDISPLAY

   :show T
}


proc Save_Options {} {
   global Options

   foreach option [list WebServer] {
       global $option
       set Options($option) [set $option]
   }

   update_Options
}

proc click-options {} {
   global Options

   load_Options

   foreach option [list WebServer] {
       global $option
       set $option $Options($option)
   }
}


#    The Options are experiment specific and normally used by SYS
#    However the same code as for the session preferences is used which allows for wider use in the future.

proc get_Option_file {} {
    global env

#    look for the Option file
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf r]} m]
    if {$z != 0} then {
        set z [catch {set f [open $pf w]} m]
        if {$z != 0} then {
            insert-log "Unable to access $pf - $m"
            return 1
        }
        close $f
        return {}
    }

#    and read it
    set lines [split [read $f] "\n"]
    close $f

    return $lines
}

proc put_Option_file {lines} {
   global env

#    look for the Option file
    set pd $env(EXPERIMENTBASE)
    if {![file exists $pd]} then {
         insert-log "$pd does not exist - Join an experiment!"
         return {}
    }
    if {![file isdirectory $pd]} then {
         insert-log "$pwd is not useable - it is not a directory"
         return {}
    }
    set pf [file join $pd MidasOptions]
    set z [catch {set f [open $pf w]} m]
    if {$z != 0} then {
        insert-log "Unable to access $pf - $m"
        return 1
    }

    foreach item $lines {
      if {$item != ""} {puts $f $item}
    }

    close $f
    return 0
}

#    save all Options for the current task
proc update_Options {} {
   global env Options

   set lines [get_Option_file]
   set key [Get_My_Name]
   set range [find-keyword $lines $key]
   if {[llength $range] > 0} then {
       set lines [lreplace $lines [first $range] [expr [second $range] + 1]]
   }

   lappend lines "## $key"
   foreach item [array names Options] {
       lappend lines "$item $Options($item)"
   }
   lappend lines "##"

   set z [put_Option_file $lines]
   return $z
}

#    load all Options for the current task
proc load_Options {} {
   global env Options

   set lines [get_Option_file]
   set key [Get_My_Name]
   set range [find-keyword $lines $key]

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           set Options([head $item]) [tail $item]
       }
   }
}

#    return task Options
proc get_Options {task} {
   global env

   set lines [get_Option_file]
   set key $task
   set range [find-keyword $lines $key]

   set Options {}

   if {[llength $range] > 1} then {
       set stuff [lrange $lines [expr [first $range]+1] [second $range]]
       foreach item $stuff {
           lappend Options [list [first $item] [second $item]]
       }
   } else {
       lappend Options [list Options.filename  "[file join $env(HOME) MidasOptions Options]"]
   }
   return $Options
}

